# define concrete transportation model in pyomo and solve it
from pyomo.environ import *
from pyomo.opt import SolverStatus, TerminationCondition
import pyomo.opt

def create_transportation_model(P = [], Q = [], a = {}, b = {}, c = {}):
    model = ConcreteModel()
    model.x = Var(P, Q, within = NonNegativeReals)
    def objective_function(model):
        return sum(c[i,j] * model.x[i,j] for i in P for j in Q)
    model.obj = Objective(rule = objective_function)
    def production_constraint(model, i):
        return sum(model.x[i,j] for j in Q) <= a[i]
    model.production = Constraint(P, rule = production_constraint)
    def demand_constraint(model, j):
        return sum(model.x[i,j] for i in P) >= b[j]
    model.demand = Constraint(Q, rule = demand_constraint)
    return model

# main
P = [1,2]
Q = [1]
a = {1:2,2:2}
b = {1:1}
c = {(1,1):1, (2,1):2}
instance = create_transportation_model(P,Q,a,b,c)
solver = pyomo.opt.SolverFactory('cplexamp', solver_io = 'nl')
if solver is None:
    raise RuntimeError('solver not found')
solver.options['timelimit'] = 60
results = solver.solve(instance, keepfiles = False, tee = True)
if ((results.solver.status == SolverStatus.ok) and (results.solver.termination_condition == TerminationCondition.optimal)):
    # feasible and optimal
    instance.solutions.load_from(results)
    xstar = [instance.x[i,j].value for i in P for j in Q]
    print('solved to optimality')
    print(xstar)
else:
    print('not solved to optimality')
    
